@@ -24,13 +24,23 @@
 """
 
 try:
-  from unittest2 import TestCase, main
+  import unittest2 as unittest
 except ImportError:
-  from unittest import TestCase, main
+  import unittest
 import libdiscid
-from libdiscid.discid import DiscError
+from libdiscid import DiscError
+
+# as long as Python 3.2 is supported, hack around the missing u
+try:
+  u = unicode
+except NameError:
+  u = str
+
+class TestLibDiscId(unittest.TestCase):
+  def test_version(self):
+    self.assertIsNotNone(libdiscid.__version__)
+    self.assertIsNotNone(libdiscid.__discid_version__)
 
-class TestLibDiscId(TestCase):
   def test_default_device(self):
     self.assertIsNotNone(libdiscid.DEFAULT_DEVICE)
 
@@ -39,9 +49,26 @@ class TestLibDiscId(TestCase):
 
   def test_features(self):
     self.assertIsNotNone(libdiscid.FEATURES)
+    self.assertIsNotNone(libdiscid.FEATURE_READ)
+    self.assertIsNotNone(libdiscid.FEATURE_MCN)
+    self.assertIsNotNone(libdiscid.FEATURE_ISRC)
+    self.assertIsNotNone(libdiscid.FEATURES_MAPPING)
 
+  @unittest.skipIf(libdiscid.FEATURES_MAPPING[libdiscid.FEATURE_READ] not in
+                   libdiscid.FEATURES, 'not available on this platform')
   def test_read_fail(self):
-    self.assertRaises(DiscError, libdiscid.read, u'/does/not/exist')
+    self.assertRaises(DiscError, libdiscid.read, u('/does/not/exist'))
+
+  def test_read_None(self):
+    try:
+      libdiscid.read()
+    except (DiscError, NotImplementedError):
+      pass
+
+  @unittest.skipIf(libdiscid.FEATURES_MAPPING[libdiscid.FEATURE_READ] in
+                   libdiscid.FEATURES, 'available on this platform')
+  def test_read_not_implemented(self):
+    self.assertRaises(NotImplemented, libdiscid.read)
 
   def test_put(self):
     first = 1
@@ -49,11 +76,13 @@ class TestLibDiscId(TestCase):
     sectors = 258725
     offsets = (150, 17510, 33275, 45910, 57805, 78310, 94650,109580, 132010,
                149160, 165115, 177710, 203325, 215555, 235590)
-    disc_id = 'TqvKjMu7dMliSfmVEBtrL7sBSno-'
-    freedb_id = 'b60d770f'
+    disc_id = u('TqvKjMu7dMliSfmVEBtrL7sBSno-')
+    freedb_id = u('b60d770f')
+    toc = u(' ').join(map(u, [first, last, sectors] + list(offsets)))
 
     disc = libdiscid.put(first, last, sectors, offsets)
     self.assertIsNotNone(disc)
+    self.assertIsNone(disc.device)
 
     self.assertEqual(disc.id, disc_id)
     self.assertEqual(disc.freedb_id, freedb_id)
@@ -63,20 +92,21 @@ class TestLibDiscId(TestCase):
     self.assertEqual(disc.sectors, sectors)
     self.assertEqual(disc.pregap, offsets[0])
     self.assertEqual(disc.leadout_track, sectors)
+    self.assertEqual(disc.toc, toc)
 
     self.assertEqual(len(disc.track_offsets), len(offsets))
-    for l, r in zip(disc.track_offsets, offsets):
-      self.assertEqual(l, r)
+    for read_offset, expected_offset in zip(disc.track_offsets, offsets):
+      self.assertEqual(read_offset, expected_offset)
 
     # ISRCs are not available if one calls put
-    if u"isrc" in libdiscid.FEATURES:
+    if libdiscid.FEATURES_MAPPING[libdiscid.FEATURE_ISRC] in libdiscid.FEATURES:
       self.assertEqual(len(disc.track_isrcs), 15)
-      for l in disc.track_isrcs:
-        self.assertEqual(l, u'')
+      for read_isrc in disc.track_isrcs:
+        self.assertEqual(read_isrc, u(''))
 
     # MCN is not available if one calls put
-    if u"mcn" in libdiscid.FEATURES:
-      self.assertEqual(disc.mcn, u'')
+    if libdiscid.FEATURES_MAPPING[libdiscid.FEATURE_MCN] in libdiscid.FEATURES:
+      self.assertEqual(disc.mcn, u(''))
 
   def test_put_fail_1(self):
     # !(first < last)
@@ -112,6 +142,14 @@ class TestLibDiscId(TestCase):
     last = 100
     self.assertRaises(DiscError, libdiscid.put, first, last, sectors, offsets)
 
+  def test_sectors_to_seconds(self):
+    # there are 75 sectors per second
+    for sectors in range(38):
+      # round down for sectors in the first half
+      self.assertEqual(libdiscid.sectors_to_seconds(sectors), 0)
+    for sectors in range(38, 76):
+      # round up for sectors in the second half
+      self.assertEqual(libdiscid.sectors_to_seconds(sectors), 1)
 
 if __name__ == '__main__':
-  main()
+  unittest.main()
